package cd.casic.ci.process.api.process.enums;

import cd.casic.ci.common.pipeline.enums.VersionStatus;
import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.process.api.process.enums
 * @Project：ops-pro
 * @name：OperationLogType
 * @Date：2025/03/25 10:10
 * @Filename：OperationLogType
 * @description：Todo
 */
@Getter
@AllArgsConstructor
public enum OperationLogType {
    CREATE_PIPELINE_AND_DRAFT("创建流水线首次保存草稿：「创建了草稿」"),
    CREATE_DRAFT_VERSION("编辑流水线生成草稿：「从 P1.T2.0 创建了草稿」"),
    UPDATE_DRAFT_VERSION("修改草稿保存后：「修改了草稿」"),
    CREATE_BRANCH_VERSION("新增分支版本：「新增版本 P1.T2.0」"),
    UPDATE_BRANCH_VERSION("修改分支版本：「修改版本 P1.T2.0」"),
    RELEASE_MASTER_VERSION("正式版本完成时：「发布版本 P1.T2.0」"),
    DISABLE_PIPELINE("禁用流水线时：「禁用了流水线」"),
    ENABLE_PIPELINE("启用流水线时：「启用了流水线」"),
    ADD_PIPELINE_OWNER("添加流水线成员时：「添加 xxx,yyy 为执行者」"),
    ADD_PIPELINE_TO_GROUP("将流水线添加到流水线组时：「添加到流水线组 a」"),
    MOVE_PIPELINE_OUT_OF_GROUP("将流水线移出流水线组时：「从流水线组 a 中移出」"),
    UPDATE_PIPELINE_SETTING("修改流水线设置：「修改配置新增版本 P2.T1.2」"),
    RESET_RECOMMENDED_VERSION_BUILD_NO("重置流水线推荐版本号：「重置流水线推荐版本号为 xxx」"),
    NORMAL_SAVE_OPERATION("普通保存操作");

    private final String description;
    public static OperationLogType parseType(String type) {
        if (type == null || type.isBlank()) return NORMAL_SAVE_OPERATION;
        try {
            return OperationLogType.valueOf(type);
        } catch (Exception ignore) {
            return NORMAL_SAVE_OPERATION;
        }
    }

    public static OperationLogType fetchType(VersionStatus versionStatus) {
        if (versionStatus == null) return NORMAL_SAVE_OPERATION;
        switch (versionStatus) {
            case COMMITTING:
                return CREATE_PIPELINE_AND_DRAFT;
            case BRANCH:
                return CREATE_BRANCH_VERSION;
            case RELEASED:
                return RELEASE_MASTER_VERSION;
            default:
                return NORMAL_SAVE_OPERATION;
        }
    }
}
