package cd.casic.ci.process.api.process.utils;


import cd.casic.ci.log.scm.dal.pojo.CodeP4Repository;
import cd.casic.ci.log.scm.dal.pojo.CodeSvnRepository;
import cd.casic.ci.log.scm.dal.pojo.Repository;
import cd.casic.ci.ticket.dal.enums.CredentialType;
import cd.casic.framework.commons.util.encrypt.DHUtil;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;

import java.util.Base64;
import java.util.List;
import java.util.Optional;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.process.api.process.utils
 * @Project：ops-pro
 * @name：CredentialUtils
 * @Date：2025/03/27 14:35
 * @Filename：CredentialUtils
 * @description：Todo
 */
@Slf4j
public class CredentialUtils {

    public static Credential getCredential(Repository repository, List<String> credentials, CredentialType credentialType) {
        if (repository instanceof CodeSvnRepository && ((CodeSvnRepository) repository).getSvnType() == CodeSvnRepository.SVN_TYPE_HTTP) {
            return switch (credentialType) {
                case USERNAME_PASSWORD -> {
                    if (credentials.size() <= 1) {
                        log.warn("Fail to get the username of the svn repo {}", repository);
                        yield new Credential(repository.getUserName(), credentials.get(0), null);
                    } else {
                        yield new Credential(credentials.get(0), credentials.get(1), null);
                    }
                }
                case TOKEN_USERNAME_PASSWORD -> {
                    yield new Credential(
                            credentials.get(0),
                            Optional.of(credentials.get(1)).orElse(""),
                            Optional.of(credentials.get(2)).orElse(""),
                            null
                    );
                }
                default -> new Credential(repository.getUserName(), credentials.get(0), null);
            };
        } else if (repository instanceof CodeSvnRepository && credentialType == CredentialType.TOKEN_SSH_PRIVATEKEY) {
            return new Credential(
                    credentials.get(0),
                    Optional.of(credentials.get(1)).orElse(""),
                    repository.getUserName(),
                    Optional.of(credentials.get(2)).orElse("")
            );
        } else if (repository instanceof CodeP4Repository && credentialType == CredentialType.USERNAME_PASSWORD) {
            return new Credential(credentials.get(0), "", credentials.get(1));
        } else {
            String privateKey = credentials.get(0);
            String passPhrase = credentials.size() > 1 && !credentials.get(1).isEmpty() ? credentials.get(1) : null;
            return new Credential(repository.getUserName(), privateKey, passPhrase);
        }
    }

    public static String decode(String encode, String publicKey, byte[] privateKey) {
        if (!StringUtils.hasText(encode)) return "";
        byte[] decodedBytes = Base64.getDecoder().decode(encode);
        byte[] publicKeyBytes = Base64.getDecoder().decode(publicKey);
        return new String(DHUtil.decrypt(decodedBytes, publicKeyBytes, privateKey));
    }

    @Data
    public static class Credential {
        private String username;
        private String privateKey;
        private String passPhrase;
        private String svnToken;
        private CredentialType credentialType;

        public Credential(String username, String privateKey, String passPhrase) {
            this.username = username;
            this.privateKey = privateKey;
            this.passPhrase = passPhrase;
        }

        public Credential(String svnToken, String username, String privateKey, String passPhrase) {
            this.svnToken = svnToken;
            this.username = username;
            this.privateKey = privateKey;
            this.passPhrase = passPhrase;
        }
    }
}
