package cd.casic.pipeline.store.api.atom;


import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.atom.MarketAtomCreateRequest;
import cd.casic.pipeline.store.pojo.atom.MarketAtomUpdateRequest;
import cd.casic.pipeline.store.pojo.common.publication.StoreProcessInfo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.web.bind.annotation.*;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;


@Tag(name = "USER_MARKET_ATOM", description = "插件市场-插件")
@RestController
@RequestMapping("/user/market/")
public interface UserAtomReleaseResource {

    @Operation(summary = "插件工作台-新增插件")
    @PostMapping("/desk/atom/")
    CommonResult<String> addMarketAtom(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "插件市场工作台-新增插件请求报文体", required = true)
            @Valid
            MarketAtomCreateRequest marketAtomCreateRequest
    );

    @Operation(summary = "插件工作台-升级插件")
    @PutMapping("/desk/atom/")
    CommonResult<String> updateMarketAtom(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目代码", required = true)
            @RequestParam("projectCode")
            String projectCode,
            @Parameter(description = "插件市场工作台-新增插件请求报文体", required = true)
            MarketAtomUpdateRequest marketAtomUpdateRequest
    );

    @Operation(summary = "根据插件版本ID获取插件版本进度")
    @GetMapping("/desk/atom/release/process/{atomId}")
    CommonResult<StoreProcessInfo> getAtomReleaseProcess(
            @Parameter(description = "插件版本ID", required = true)
            @PathVariable("atomId")
            String atomId
    );
}