package cd.casic.pipeline.store.api.template;

import cd.casic.ci.common.pipeline.Model;
import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.template.InstallTemplateReq;
import cd.casic.pipeline.store.pojo.template.MarketTemplateResp;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.web.bind.annotation.*;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;

@Tag(name = "SERVICE_MARKET_TEMPLATE", description = "服务端-模板")
@RestController
@RequestMapping("/service/market/template")
public interface ServiceTemplateResource {

    @Operation(summary = "模版市场搜索模版")
    @GetMapping("/list/")
    CommonResult<MarketTemplateResp> list(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId
    );

    @Operation(summary = "安装模板到项目")
    @PostMapping("/template/install")
    CommonResult<Boolean> installTemplate(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "安装模板到项目请求报文体", required = true)
            InstallTemplateReq installTemplateReq
    );

    @Operation(summary = "校验模板内组件可见范围")
    @GetMapping("/{templateCode}/validate")
    CommonResult<Boolean> validateUserTemplateComponentVisibleDept(
            @Parameter(description = "用户", required = true)
            @RequestParam("userId")
            String userId,
            @Parameter(description = "标识", required = true)
            @PathVariable("templateCode")
            String templateCode,
            @Parameter(description = "项目", required = true)
            @RequestParam("projectCode")
            String projectCode
    );

    @Operation(summary = "校验流水线模型内组件可见范围")
    @PostMapping("/{projectCode}/verification")
    CommonResult<Boolean> validateModelComponentVisibleDept(
            @Parameter(description = "用户", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "流水线模型-阶段", required = true)
            @Valid
            Model model,
            @Parameter(description = "项目", required = true)
            @PathVariable("projectCode")
            String projectCode
    );
}