package cd.casic.ci.ticket.api;


import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.ci.ticket.dal.pojo.Credential;
import cd.casic.ci.ticket.dal.pojo.CredentialCreate;
import cd.casic.ci.ticket.dal.pojo.CredentialInfo;
import cd.casic.ci.ticket.dal.pojo.CredentialUpdate;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.security.Permission;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.*;


@RestController
@Tag(name = "服务-凭据资源")
@RequestMapping("/service/credentials")
public interface ServiceCredentialApi {

    @Operation(summary = "新增凭据")
    @PostMapping("/{projectId}/")
    CommonResult<Boolean> create(
            @Parameter(description = "User ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "Project ID", required = true)
            @PathVariable("projectId")
            String projectId,
            @Parameter(description = "Credential", required = true)
            CredentialCreate credential
    );

    @Operation(summary = "其他服务获取凭据")
    @GetMapping("/{projectId}/{credentialId}/")
    CommonResult<CredentialInfo> get(
            @Parameter(description = "Project ID", required = true)
            @PathVariable("projectId")
            String projectId,
            @Parameter(description = "Credential ID", required = true)
            @PathVariable("credentialId")
            String credentialId,
            @Parameter(description = "Base64-encoded public key for encryption", required = true)
            @RequestParam("publicKey")
            String publicKey
    );

    @Operation(summary = "检查凭据是否存在")
    @PutMapping("/{projectId}/{credentialId}/")
    CommonResult<Boolean> check(
            @Parameter(description = "Project ID", required = true)
            @RequestParam("projectId")
            String projectId,
            @Parameter(description = "Credential ID", required = true)
            @RequestParam("credentialId")
            String credentialId
    );

    @Operation(summary = "其他服务获取凭据列表")
    @GetMapping("/{projectId}/")
    CommonResult<PageResult<Credential>> list(
            @Parameter(description = "Project ID", required = true)
            @PathVariable("projectId")
            String projectId,
            @Parameter(description = "Page number", required = false, example = "1")
            @RequestParam("page")
            Integer page,
            @Parameter(description = "Number of items per page", required = false, example = "20")
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "获取拥有对应权限凭据列表")
    @GetMapping("/{projectId}/hasPermissionList")
    CommonResult<PageResult<Credential>> hasPermissionList(
            @Parameter(description = "User ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "Project ID", required = true)
            @PathVariable("projectId")
            String projectId,
            @Parameter(description = "List of credential types, separated by commas", required = false, example = "")
            @RequestParam("credentialTypes")
            String credentialTypesString,
            @Parameter(description = "Corresponding permission", required = true, example = "")
            @RequestParam("permission")
            Permission permission,
            @Parameter(description = "Page number", required = false, example = "1")
            @RequestParam("page")
            Integer page,
            @Parameter(description = "Number of items per page", required = false, example = "20")
            @RequestParam("pageSize")
            Integer pageSize,
            @Parameter(description = "Keyword", required = false)
            @RequestParam("keyword")
            String keyword
    );

    @Operation(summary = "编辑凭据")
    @PutMapping("/projects/{projectId}/credentials/{credentialId}/")
    CommonResult<Boolean> edit(
            @Parameter(description = "User ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "Project ID", required = true)
            @PathVariable("projectId")
            String projectId,
            @Parameter(description = "Credential ID", required = true)
            @PathVariable("credentialId")
            String credentialId,
            @Parameter(description = "Credential", required = true)
            CredentialUpdate credential
    );
}
